<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Utils\{TenantUtils, VitalPBXApiUtils};
use Illuminate\Http\Request;

class AgentController extends Controller
{
    public function __invoke(Request $request)
    {
        $user = auth()->user()->load('roles', 'systemPermission');

        $tenantId = TenantUtils::getTenantId($request, $user);

        $agents = $this->getAgents($tenantId);
    
        if ($user->isSuperAdmin() || is_null($user->systemPermission)) {
            $response = $agents;
        } else {
            $response = $this->getSystemPermissionExtensions($tenantId, $agents, $user->systemPermission->agents ?? []);
        }
        
        return ['data' => ['extensions' => $response]];
    }

    private function getAgents($tenantId)
    {
        $response = VitalPBXApiUtils::request('extensions', $tenantId);

        return is_array($response) ? collect($response['data']) : collect();
    }

    private function getSystemPermissionExtensions($tenantId, $agents, $collection)
    {
        $items = collect();

        foreach($collection as $agent) {
            $items->push([
                'extension' => $agent['extension'],
                'name' => $agents->where('extension', $agent['extension'])->first()->name ?? ''
            ]);
        }

        return $items->all();
    }
}
