<?php

namespace App\Http\Controllers\Api;

use App\{Models\CampaignResult, Models\TimezoneSetting};
use App\Exports\ReportExport;
use App\Http\Controllers\Controller;
use App\Http\Requests\ReportRequest;
use App\Utils\ReportUtils;

class AgentReportController extends Controller
{
    public function __invoke(ReportRequest $request)
    {
        $validatedData = $request->validated();

        $timezone = TimezoneSetting::getSettings($validatedData['tenant_id'])->timezone;

        $startDate = ReportUtils::transformTime($validatedData['start_date'], $timezone);
        $endDate = ReportUtils::transformTime($validatedData['end_date'], $timezone, true);

        $collection = CampaignResult::leftJoin('dispositions', 'dispositions.id', '=', 'campaign_results.disposition_id')
            ->leftJoin('users', 'users.id', '=', 'campaign_results.agent_id')
            ->selectRaw("users.id AS agent_id, users.full_name AS agent, users.extension AS extension,
                COUNT(CASE WHEN dispositions.category = 'positive' THEN 1 else NULL END) AS positive,
                COUNT(CASE WHEN dispositions.category = 'call_later' THEN 1 else NULL END) AS call_later,
                COUNT(CASE WHEN dispositions.category = 'negative' THEN 1 else NULL END) AS negative,
                COUNT(dispositions.id) AS attended_contacts")
            ->whereBetween('campaign_results.created_at', [$startDate, $endDate])

            ->when($validatedData['campaign_id'], function ($query, $campaign) {
                return $query->where('campaign_id', $campaign);
            })
            ->when($validatedData['agents'] ?? null, function ($query, $agents) {
                return $query->whereIn('agent_id', $agents);
            })
            ->groupBy(['users.id', 'users.full_name', 'users.extension'])
            ->get();

        if (isset($validatedData['format'])) {
            return ReportExport::export(
                $collection, 'agent_summary', $validatedData['format'],
                $validatedData['start_date'], $validatedData['end_date']
            );
        }

        return ['data' => $collection];
    }
}

