<?php

namespace App\Http\Controllers\Api;

use App\{Models\Campaign, Models\CampaignResult, Models\TimezoneSetting};
use App\Exports\ReportExport;
use App\Http\Controllers\Controller;
use App\Http\Requests\ReportRequest;
use App\Http\Resources\AgentReportDetail as AgentReportDetailResource;
use App\Utils\ReportUtils;

class AgentReportDetailController extends Controller
{
    public function __invoke(ReportRequest $request)
    {
        $validatedData = $request->validated();

        $timezone = TimezoneSetting::getSettings($validatedData['tenant_id'])->timezone;
        
        $startDate = ReportUtils::transformTime($validatedData['start_date'], $timezone);
        $endDate = ReportUtils::transformTime($validatedData['end_date'], $timezone, true);

        $collection = CampaignResult::leftJoin('dispositions', 'dispositions.id', '=', 'campaign_results.disposition_id')
            ->leftJoin('users', 'users.id', '=', 'campaign_results.agent_id')
            ->leftJoin('contacts', 'contacts.id', '=', 'campaign_results.contact_id')
            ->selectRaw("users.full_name AS agent, users.extension, contacts.phone_1 AS contacts_phone_1,
                contacts.phone_2 AS contacts_phone_2, contacts.phone_3 AS contacts_phone_3,
                contacts.extra_fields AS contacts_contact_info, dispositions.name AS disposition,
                campaign_results.phone_1, campaign_results.phone_2, campaign_results.phone_3,
                campaign_results.contact_info, dispositions.category AS category, campaign_results.created_at,
                campaign_results.updated_at, campaign_results.call_resolution_time, campaign_results.reason, fields")
            ->whereBetween('campaign_results.created_at', [$startDate, $endDate])
            ->when($validatedData['campaign_id'], function ($query, $campaign) {
                return $query->where('campaign_id', $campaign);
            })
            ->when($validatedData['agents'] ?? null, function ($query, $agents) {
                return $query->whereIn('agent_id', $agents);
            })
            ->when((isset($validatedData['category']) && $validatedData['category'] != 'all'), function ($query) use ($validatedData) {
                return $query->where('dispositions.category', $validatedData['category']);
            })
            ->when($validatedData['contact'] ?? null, function ($query, $contact) {
                $contact = $contact.'%';
                return $query->where('campaign_results.phone_1', 'like', $contact)
                    ->orWhere('campaign_results.phone_2', 'like', $contact)
                    ->orWhere('campaign_results.phone_3', 'like', $contact)
                    ->OrWhere('contacts.phone_1', 'like', $contact)
                    ->orWhere('contacts.phone_2', 'like', $contact)
                    ->orWhere('contacts.phone_3', 'like', $contact);
            });
            
        //return report for download if format parameter exists
        //else return json data
        if (isset($validatedData['format'])) {
            $campaign = Campaign::with('form')->findOrFail($validatedData['campaign_id']);

            return ReportExport::export(
                $collection->get(), 'agent_detail', $validatedData['format'],
                $validatedData['start_date'], $validatedData['end_date'],
                $campaign
            );
        }

        return AgentReportDetailResource::collection($collection->paginate($validatedData['per_page'] ?? 10));
    }
}

