<?php

namespace App\Http\Controllers\Api;

use App\{Models\AppSetting};
use App\Http\Controllers\Controller;
use App\Services\LicenseService;
use App\Utils\{SettingsUtils, TenantUtils, VitalPBXApiUtils};
use Illuminate\Http\Request;

class AuthController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth:api', ['except' => ['login']]);
    }

    public function login(Request $request)
    {
        $credentials = $request->only(['name', 'password']);

        $token = auth()->attempt($credentials);

        if (!$token) {
            return response()->json(['error' => 'Unauthorized'], 401);
        }

        return $this->respondWithToken($token, $request);
    }

    public function me(Request $request)
    {
        return response()->json($this->getUser($request, true));
    }

    public function logout()
    {
        auth()->logout();

        return ['message' => 'Successfully logged out'];
    }

    protected function respondWithToken($token, $request)
    {
        $response = [
            'access_token' => $token,
            'token_type' => 'bearer',
            'expires_in' => auth()->factory()->getTTL() * 60,
            'user' => $this->getUser($request, true)
        ];

        return response()->json($response);
    }
    
    protected function getUser($request, $getPbx = null)
    {
        $user = auth()->user()->load('roles.permissions', 'agentProfile.pauses');
        $tenantId = TenantUtils::getTenantId($request, $user);
        $user->tenant_prefix = $this->getTenantPrefix($user, $tenantId);
        $user->version = (new LicenseService())->getVersion();
        
        if ($user->hasRole('agent')) {
            $user->load('settings');
             
            $user->device = VitalPBXApiUtils::request("devices/{$user->device_id}", $user->tenant_id);
            
            $user->device = is_array($user->device) ? $user->device['data'] : null;
            
            $user->ice_servers = AppSetting::iceServerSettings()->first();

            $user->profile = $user->agentProfile;
        }
        
        if (isset($getPbx)) {
            $pbxSettings = SettingsUtils::getPbxSettings();
        
            $user->pbx = [
                'host' => $pbxSettings['host'],
                'port' => $pbxSettings['port'],
                'secure' => $pbxSettings['secure'],
                'pjsip_port' => $pbxSettings['pjsip_port'],
                'wss_port' => $pbxSettings['wss_port']
            ];
        }
        
        return $user;
    }
    
    protected function getTenantPrefix($user, $tenantId = null)
    {
        $tenantId = $tenantId ?? $user->tenant_id;
        
        return $tenantId == '1' ? null : "T{$tenantId}_";
    }
}
