<?php

namespace App\Http\Controllers\Api;

use App\{Models\Campaign, Models\TimezoneSetting, Models\User};
use App\Exports\ReportExport;
use App\Http\Controllers\Controller;
use App\Http\Requests\ReportRequest;
use App\Utils\ReportUtils;
use Illuminate\Support\Facades\DB;

class CallByAgentController extends Controller
{
    public function __invoke(ReportRequest $request)
    {
        $validatedData = $request->validated();

        $timezone = TimezoneSetting::getSettings($validatedData['tenant_id'])->timezone;

        $startDate = ReportUtils::transformTime($validatedData['start_date'], $timezone);
        $endDate = ReportUtils::transformTime($validatedData['end_date'], $timezone, true);

        $campaign = Campaign::where('id', $validatedData['campaign_id'])->select('name', 'id')->get();
        $agents = User::whereIn('extension', $validatedData['agents'])->select('full_name', 'extension')->get();

        $collection = DB::connection('asterisk')->table('queues_log')
            ->where('event', 'OUTCALL')
            ->selectRaw("agent AS extension, data3 AS campaign,
                COUNT(*) AS total_calls, SEC_TO_TIME(SUM(data2)) AS total_calls_length,
                SEC_TO_TIME(AVG(data2)) AS avg_calls_length
            ")
            ->whereRaw('time BETWEEN ? AND ?', [$startDate, $endDate])
            ->whereIn('agent', $validatedData['agents'])
            ->where('data3', $validatedData['campaign_id'])
            ->groupBy('agent')
            ->get();

        $totalCalls = $collection->sum('total_calls');

        $collection->transform(function ($item) use ($totalCalls, $campaign, $agents) {
            $fullName = $agents->where('extension', $item->extension)->first()->full_name ?? '';
            $item->agent = $fullName;
            $item->campaign = $campaign->where('id', $item->campaign)->first()->name ?? $item->campaign;
            $item->total_calls_percent = ReportUtils::calculatePercent($item->total_calls, $totalCalls);

            return $item;
        });

        if (isset($validatedData['format'])) {
            return ReportExport::export(
                $collection, 'calls_summary', $validatedData['format'],
                $validatedData['start_date'], $validatedData['end_date']
            );
        }

        return ['data' => $collection];
    }
}
