<?php

namespace App\Http\Controllers\Api;

use App\Exports\ReportExport;
use App\Http\Controllers\Controller;
use App\Http\Requests\ReportRequest;
use App\Models\TimezoneSetting;
use App\Utils\ReportUtils;
use Illuminate\Support\Facades\DB;

class CallByAgentDetailController extends Controller
{
    public function __invoke(ReportRequest $request)
    {
        $validatedData = $request->validated();

        $timezone = TimezoneSetting::getSettings($validatedData['tenant_id'])->timezone;

        $startDate = ReportUtils::transformTime($validatedData['start_date'], $timezone);
        $endDate = ReportUtils::transformTime($validatedData['end_date'], $timezone, true);

        $collection = DB::connection('asterisk')->table('queues_log')
            ->where('event', 'OUTCALL')
            ->selectRaw("CONVERT_TZ(time, '+00:00', ?) AS date, data1 AS call_status, data3 as campaign, SEC_TO_TIME(data2) AS duration,
              data4 AS customer", [ReportUtils::getOffset($timezone)])
            ->whereRaw('time BETWEEN ? AND ?', [$startDate, $endDate])
            ->whereIn('agent', $validatedData['agents'])
            ->where('data3', $validatedData['campaign_id'])
            ->orderBy('time');

        $collection = isset($validatedData['format']) ? $collection->get() : $collection->paginate($validatedData['per_page'] ?? 10);

        $collection->transform(function ($item) {
            $item->call_status = __(strtolower($item->call_status));

            return $item;
        });

        if (isset($validatedData['format'])) {
            return ReportExport::export(
                $collection, 'calls_detail', $validatedData['format'],
                $validatedData['start_date'], $validatedData['end_date']
            );
        }

        return $collection;
    }
}
