<?php

namespace App\Http\Controllers\Api;

use App\{Models\CampaignResult, Models\TimezoneSetting};
use App\Http\Controllers\Controller;
use App\Http\Requests\ReportRequest;
use App\Utils\ReportUtils;

class CampaignResultByDispositionController extends Controller
{
    public function __invoke(ReportRequest $request)
    {
        $validatedData = $request->validated();

        $timezone = TimezoneSetting::getSettings($validatedData['tenant_id'])->timezone;

        $startDate = ReportUtils::transformTime($validatedData['start_date'], $timezone);
        $endDate = ReportUtils::transformTime($validatedData['end_date'], $timezone, true);

        $collection = CampaignResult::join('dispositions', 'dispositions.id', '=', 'campaign_results.disposition_id')
            ->where('campaign_id', $validatedData['campaign_id'])
            ->whereBetween('campaign_results.created_at', [$startDate, $endDate])
            ->when($request->input('dispositions'), function ($query) use ($validatedData) {
                $query->whereIn('campaign_results.disposition_id', $validatedData['dispositions']);
            })
            ->selectRaw("dispositions.category, dispositions.name, COUNT(*) AS count")
            ->groupBy('dispositions.category', 'dispositions.name')
            ->get();

        $categories = collect();

        $total = $collection->sum('count');

        $collection->groupBy('category')->each(function ($item) use ($categories, $total) {
            $count = $item->sum('count');

            $categories->push([
               'category' => $item->first()->category,
               'count' => $count,
               'percent' => ReportUtils::calculatePercent($count, $total)
            ]);
        });

        return [
            'data' => [
                'categories' => $categories,
                'dispositions' => $collection->groupBy('category')
            ]
        ];
    }
}
