<?php

namespace App\Http\Controllers\Api;

use App\{Models\Campaign, Models\CampaignContact, Models\User};
use App\Http\Controllers\Controller;
use App\Utils\TenantUtils;
use Illuminate\Http\Request;

class DashboardController extends Controller
{
    public function __invoke(Request $request)
    {
        $tenantId = TenantUtils::getTenantId($request);

        $agents = User::byTenant($tenantId)->agents()->count();

        $attendedContacts = CampaignContact::whereHas('campaign', function ($query) use ($tenantId) {
            $query->where('tenant_id', $tenantId);
        })
        ->where('status', 'finished')->count();

        $campaigns = Campaign::byTenant($tenantId)->selectRaw('status, COUNT(*) AS count')->groupBy('status')->get();

        $activeCampaigns = $campaigns->where('status', 'active')->first()['count'] ?? 0;

        $inactiveCampaigns = $campaigns->where('status', 'inactive')->first()['count'] ?? 0;

        $finishedCampaigns = $campaigns->where('status', 'finished')->first()['count'] ?? 0;

        return [
            'data' => [
                'agents' => $agents,
                'attended_contacts' => $attendedContacts,
                'active_campaigns' => $activeCampaigns,
                'inactive_campaigns' => $inactiveCampaigns,
                'finished_campaigns' => $finishedCampaigns
            ]
        ];
    }
}
