<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\Forms\StoreForm;
use App\Http\Resources\Form as FormResource;
use App\Models\Form;
use App\Utils\TenantUtils;
use Illuminate\Http\Request;

class FormController extends Controller
{
    public function index(Request $request)
    {
        $this->authorize('viewAny', Form::class);

        return FormResource::collection(Form::with('campaigns')
            ->byTenant(TenantUtils::getTenantId($request))
            ->orderBy('updated_at', 'desc')
            ->get());
    }

    public function store(StoreForm $request)
    {
        $this->authorize('create', Form::class);

        return ['data' => Form::create($request->validated())];
    }

    public function update(StoreForm $request, Form $form)
    {
        $this->authorize('updateDestroy', $form);

        // $response = $this->checkCampaigns($form, 'update');

        // if ($response) return $response;

        $form->update($request->validated());

        return ['data' => $form];
    }

    public function destroy(Request $request, Form $form)
    {
        $this->authorize('updateDestroy', $form);

        $response = $this->checkCampaigns($form, 'delete');

        if ($response) return $response;

        $form->delete();

        return response()->json([], 204);
    }

    private function checkCampaigns($form, $action)
    {
        $message = [
            'update' => __('form_cannot_update'),
            'delete' => __('form_cannot_delete'),
        ];

        //if there are associated campaigns with the current form can't be edited
        $activeCampaigns = $form->campaigns()->active();
        if ($activeCampaigns->count() > 0) {
            return response()->json(['message' => $message[$action] . ': ' . $activeCampaigns->pluck('name')->implode(', ') ], 422);
        }
    }
}
