<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\TimezoneSetting;
use App\Utils\TenantUtils;
use Carbon\CarbonTimeZone;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\{Gate, Route};

class GeneralSettingController extends Controller
{
    public function __construct()
    {
        $installation = @file_get_contents('/var/lib/sonata/dialer/wizard.conf');
        
        if (trim($installation) === 'installed=yes') {
            $this->middleware('auth:api');
            
            //apply permission only on store action
            if (Route::currentRouteName() === 'general-settings.store') {
                $this->middleware(function ($request, $next) {
                    if (Gate::allows('general_settings')) return $next($request);
                    
                    abort(403);
                });
            }
        }
    }

    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'timezone' => 'required|timezone',
            'tenant_id' => 'required'
        ]);

        return TimezoneSetting::updateOrCreate(
            ['tenant_id' => $validatedData['tenant_id']],
            ['timezone' => $validatedData['timezone']]
        );
    }

    public function show(Request $request)
    {
        $timezoneSetting = TimezoneSetting::byTenant($this->getTenantId($request))->first();

        $timezones = collect(CarbonTimeZone::listIdentifiers());

        $timezones->transform(function ($item) {
            return ['text' => str_replace('_', ' ', $item), 'value' => $item];
        });

        return [
            'general_settings' => $timezoneSetting,
            'timezones' => $timezones
        ];
    }

    private function getTenantId($request)
    {
        if (is_null(auth()->user())) return 1;

        return TenantUtils::getTenantId($request);
    }
}
