<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\LicenseRequest;
use App\Services\LicenseService;

class LicenseController extends Controller
{
    public function getLicenseData(LicenseService $licenseService)
    {
        return ['data' => $licenseService->getLicenseData()];
    }

    public function getVersion(LicenseService $licenseService)
    {
        return ['data' => $licenseService->getVersion()];
    }

    public function deleteLicense(LicenseService $licenseService)
    {
        return ['deleted' => $licenseService->deleteLicense()];
    }

    public function activateProduct(LicenseRequest $request, LicenseService $licenseService)
    {
        $validatedData = $this->getCustomerData($request);

        return $licenseService->activateProduct($validatedData['serial'], $validatedData['customer_data']);
    }

    public function migrateProduct(LicenseRequest $request, LicenseService $licenseService)
    {
        $validatedData = $this->getCustomerData($request);

        return $licenseService->migrateProduct($validatedData['serial'], $validatedData['customer_data']);
    }

    private function getCustomerData($request)
    {
        $validatedData = $request->validated();

        $validatedData['customer_data'] = [
            'company' => $validatedData['company'], 'full_name' => $validatedData['full_name'], 'email' => $validatedData['email'],
        ];

        return $validatedData;
    }
}
