<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\PbxSettingRequest;
use App\Utils\VitalPBXApiUtils;
use Illuminate\Support\Facades\Http;

class PbxConnectionCheckController extends Controller
{
    public function __invoke(PbxSettingRequest $request)
    {
        $validatedData = $request->validated();

        if (! isset($validatedData['app_key'])) {
            $validatedData['app_key'] = VitalPBXApiUtils::checkAppKeyFile();
            
            if (! isset($validatedData['app_key'])) {
                return response()->json(['message' => __('pbx_app_key_required')], 422);
            }
        }
        
        $protocol = isset($validatedData['secure']) && (bool) $validatedData['secure'] === true ? 'https' : 'http';
        $apiPort = $validatedData['api_port'];

        $response = Http::withOptions(['verify' => false])
            ->withHeaders([
                'app-key' => $validatedData['app_key']
            ])
            ->get("{$protocol}://{$validatedData['host']}:{$apiPort}/api/v2/tenants");

        return ['is_connected' => $response->successful()];
    }
}
