<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\PbxSettingRequest;
use App\Utils\{SettingsUtils, VitalPBXApiUtils};
use Jackiedo\DotenvEditor\Facades\DotenvEditor;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;

class PbxSettingController extends Controller
{
    public function __construct()
    {
        $installation = @file_get_contents('/var/lib/sonata/dialer/wizard.conf');
        
        if (trim($installation) === 'installed=yes') {
            $this->middleware('auth:api');

            $this->middleware(function ($request, $next) {
                if (Gate::allows('pbx_settings')) return $next($request);
                
                abort(403);
            });
        }
    }
    
    public function store(PbxSettingRequest $request)
    {
        $validatedData = $request->validated();

        if (! isset($validatedData['app_key']) || $validatedData['app_key'] === '') {
            if (VitalPBXApiUtils::checkAppKeyFile()) {
                 $validatedData['app_key'] = VitalPBXApiUtils::checkAppKeyFile();
            } else {
                return response()->json(['message' => __('pbx_app_key_required')], 422);
            }
        }

        $editor = DotenvEditor::load();

        $protocol = (bool) $validatedData['secure'] === true ? 'https' : 'http';

        if ($protocol == 'https' && $validatedData['api_port'] == '3500') {
            $validatedData['api_port'] = '3501';
        } elseif ($protocol == 'http' && $validatedData['api_port'] == '3501') {
            $validatedData['api_port'] = '3500';
        }

        $editor->setKeys([
            'PBX_HOST' => $validatedData['host'],
            'PBX_PORT' => $validatedData['port'],
            'PBX_SECURE' => $validatedData['secure'],
            'PBX_PJSIP_PORT' => $validatedData['pjsip_port'],
            'PBX_WSS_PORT' => $validatedData['wss_port'],
            'PBX_API_PORT' => $validatedData['api_port'],
            'PBX_APP_KEY' => $validatedData['app_key'],
        ]);

        $editor->save();

        SettingsUtils::cacheSettings();

        return ['message' => __('pbx_settings_updated')];
    }

    public function show()
    {
        $settings = SettingsUtils::getPbxSettings();
        
        $appKey = VitalPBXApiUtils::checkAppKeyFile();
        
        $settings['app_key_exists'] = false;
        
        if ($appKey) {
            $settings['app_key_exists'] = true;
        }
        
        if ($appKey === $settings['app_key']) {
            unset($settings['app_key']);
        }
        
        return $settings;
    }
}
