<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\Role;
use Illuminate\Support\Str;

class PermissionController extends Controller
{
    public function __invoke()
    {
        $this->authorize('roles', Role::class);
        
        $user = auth()->user()->load('roles.permissions');
        
        $permissions = collect($user->roles()->first()->permissions);
        
        $campaigns = [
            'campaigns', 'forms', 'contact lists', 'dnc numbers', 'dispositions', 'queues'
        ];
        
        $users = [
            'users', 'roles', 'system permissions', 'pause profiles'
        ];

        $reports = [
            'agents report', 'dispositions report', 'calls by agent', 'broadcast campaigns report'
        ];
        
        $settings = [
            'general settings', 'ice servers', 'pbx settings'
        ];
        
        $license = [
            'license'
        ];

        $permissions->transform(function ($item) use ($campaigns, $users, $reports, $settings, $license) {
            $module = $item->module;
            
            if (in_array($module, $campaigns)) {
                $item->group = 'campaigns';
                $item->order = 1;
            } elseif (in_array($module, $users)) {
                $item->group = 'users_management';
                $item->order = 2;
            } elseif (in_array($module, $reports)) {
                $item->group = 'reports';
                $item->order = 3;
            } elseif (in_array($module, $settings)) {
                $item->group = 'settings';
                $item->order = 4;
            } elseif (in_array($module, $license)) {
                $item->group = 'license';
                $item->order = 5;
            }
            
            return $item;
        });
        
        $permissions = $permissions->reject(function ($item) {
            return is_null($item->group);
        });
        
        $groupedPermissions = collect();
        
        $permissions->groupBy('group')->transform(function ($item) use ($groupedPermissions) {
            $groupedPermissions->push([
                'group' => $item->first()->group,
                'order' => $item->first()->order,
                'permissions' => $item->transform(function ($item) {
                    $item->module = Str::slug($item->module, '_');
                    unset($item->order);
                    return $item;
                })
            ]);
        });
        
        return ['data' => $groupedPermissions->sortBy('order')->values()->all()];
    }
}
