<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\Roles\{StoreRole, UpdateRole};
use App\Http\Resources\Role as RoleResource;
use App\Models\Role;

class RoleController extends Controller
{
    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $this->authorize('roles', Role::class);

            return $next($request);
        });
    }

    public function index()
    {
        return RoleResource::collection(Role::with(['permissions', 'users'])
            ->editable($this->getUser())
            ->where('name', '!=' ,'agent')
            ->get()
        );
    }

    public function store(StoreRole $request)
    {
        $validatedData = $request->validated();

        $role = Role::create($validatedData);

        $role->permissions()->attach($validatedData['permissions']);

        return new RoleResource($role);
    }

    public function show($id)
    {
        return new RoleResource(Role::with(['permissions', 'users'])
            ->editable($this->getUser())
            ->findOrFail($id));
    }

    public function update(UpdateRole $request, $id)
    {
        $validatedData = $request->validated();
        
        $role = Role::editable($this->getUser())->findOrFail($id);

        $role->update($validatedData);

        $role->permissions()->sync($validatedData['permissions']);

        return new RoleResource($role);
    }

    public function destroy($id)
    {
        $role = Role::with('users')->editable($this->getUser())->findOrFail($id);
        
        if ($role->users()->count() > 0) return response()->json(['message' => __('role_cannot_delete')], 422);
    
        $role->delete();

        return response()->json([], 204);
    }
    
    private function getUser()
    {
        return Auth()->user()->load('roles');
    }
}
