<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\SalesforceSettingRequest;
use App\Utils\SettingsUtils;
use Illuminate\Http\Request;

class SalesforceSettingController extends Controller
{
    public function store(SalesforceSettingRequest $request)
    {
        $validatedData = $request->validated();
        
        $settings = $this->getSalesforceSettings();
        
        $envKeys = [
            'consumer_key' => 'SF_CONSUMER_KEY',
            'consumer_secret' => 'SF_CONSUMER_SECRET',
            'login_url' => 'SF_LOGIN_URL',
            'username' => 'SF_USERNAME',
            'password' => 'SF_PASSWORD',
            'security_token' => 'SF_SECURITY_TOKEN'
        ];
        
        SettingsUtils::updateSettings($validatedData, $settings, $envKeys);

        return response()->json(['message' => __('sales_settings_updated')]);
    }
    
    public function show()
    {
        return $this->getSalesforceSettings();
    }
    
    private function getSalesforceSettings()
    {
        return [
            'consumer_key' => config('forrest.credentials.consumerKey'),
            'consumer_secret' => config('forrest.credentials.consumerSecret'),
            'login_url' => config('forrest.credentials.loginUrl'),
            'username' => config('forrest.credentials.username'),
            'password' => config('forrest.credentials.password'),
            'security_token' => env('forrest.credentials.security_token')
        ];
    }
}
