<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\SystemPermission\StoreSystemPermission;
use App\Http\Resources\SystemPermission as SystemPermissionResource;
use App\Models\SystemPermission;
use App\Utils\{TenantUtils, VitalPBXApiUtils};
use Illuminate\Http\Request;

class SystemPermissionController extends Controller
{
    public function index(Request $request)
    {
        $this->authorize('systemPermission', SystemPermission::class);

        $tenantId = TenantUtils::getTenantId($request);

        $systemPermissions = SystemPermission::with('campaigns')
            ->byTenant($tenantId)
            ->get();

        $agents = VitalPBXApiUtils::request('agents', $tenantId);

        if (is_array($agents)) {
            $agents = collect($agents['data']);

            //add agent name from api
            $systemPermissions->transform(function ($item) use ($agents) {
                $items = collect();

                foreach($item->agents as $agent) {
                    $items->push([
                        'extension' => $agent['extension'],
                        'name' => $agents->where('extension', $agent['extension'])->first()->name ?? ''
                    ]);
                }

                $item->agents = $items->all();

                return $item;
            });
        }

        return ['data' => $systemPermissions];
    }
    
    public function store(StoreSystemPermission $request)
    {
        $validatedData = $request->validated();

        $this->authorize('systemPermission', SystemPermission::class);

        $systemPermission = SystemPermission::create($validatedData);

        if ($request->has('campaigns')) {
            $systemPermission->campaigns()->attach($validatedData['campaigns']);
        }
        
        return ['data' => new SystemPermissionResource($systemPermission)];
    }
    
    public function update(StoreSystemPermission $request, SystemPermission $systemPermission)
    {
        $validatedData = $request->validated();

        $this->authorize('updateDestroy', $systemPermission);
            
        $systemPermission->update($validatedData);

        $systemPermission->campaigns()->sync($validatedData['campaigns']);
        
        return ['data' => new SystemPermissionResource($systemPermission)];
    }
    
    public function destroy(Request $request, $id)
    {
        $systemPermission = SystemPermission::with('users')->findOrFail($id);

        $this->authorize('updateDestroy', $systemPermission);
            
        if ($systemPermission->users()->count() > 0) return response()->json(['message' => __('system_permission_cannot_delete')], 422);
    
        $systemPermission->delete();

        return response()->json([], 204);
    
    }
}
